define("customBlocks_grove", [
  "blocklylib"
  ,"./blockly/msg"
  ,"./blockly/constants"
  ,"./blockly/context-menus"
  ,"./blockly/io"
  ,"./blockly/colors"
  ,"./blockly/block-utils"
  ,"./blockly/variables"
  ,"./blockly/field-variable"
  ,"./blockly/generators/picaxebasic"
], function (BlocklyLib, msg, constants, contextMenus, io, colors, blockUtils, variables, FieldVariable, basicGenerator) {

var blockDefs = BlocklyLib.Blocks;

blockDefs['grove_i2c_colour_sensor'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.INPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C Colour Sensor")
        .appendField(new BlocklyLib.FieldDropdown([
          ["v2.0", "20"],
          ["v1.2", "12"]
        ]), "TYPE")
    this.appendDummyInput()
        .appendField(msg.get("GROVE_RED"))
        .appendField(new FieldVariable(variables.defaultVarName, variables.validateVarName), "VARR")
        .appendField(msg.get("GROVE_GREEN"))
        .appendField(new FieldVariable(variables.defaultVarName2, variables.validateVarName), "VARG")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.appendDummyInput()
        .appendField(msg.get("GROVE_BLUE"))
        .appendField(new FieldVariable(variables.defaultVarName3, variables.validateVarName), "VARB")
         .appendField(msg.get("GROVE_CLEAR"))
         .appendField(new FieldVariable(variables.defaultVarName4, variables.validateVarName), "VARW")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('I2C Colour Sensor (101020041)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  getVars: function () {
    return variables.getAll(this, ["VARR", "VARG", "VARB", "VARW"]);
  },
  renameVar: function (oldName, newName) {
    return variables.rename(this, ["VARR", "VARG", "VARB", "VARW"], oldName, newName);
  },
};

blockDefs['grove_rtc'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.INPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C RTC")
        .appendField(new BlocklyLib.FieldDropdown([
          ["Time" + " hh:mm:ss", "Time"],
          ["Date" + " dd:mm:yy", "Date"]
        ]), "TYPE")
    this.appendDummyInput()
        .appendField(new FieldVariable(variables.defaultVarName, variables.validateVarName), "VAR1")
        .appendField(":")
        .appendField(new FieldVariable(variables.defaultVarName2, variables.validateVarName), "VAR2")
        .appendField(":")
        .appendField(new FieldVariable(variables.defaultVarName3, variables.validateVarName), "VAR3")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('RTC (101020013)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  getVars: function () {
    return variables.getAll(this, ["VAR1", "VAR2", "VAR3"]);
  },
  renameVar: function (oldName, newName) {
    return variables.rename(this, ["VAR1", "VAR2", "VAR3"], oldName, newName);
  },
};

blockDefs['grove_nunchuck'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.INPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C Nunchuck")
    this.appendDummyInput()
        .appendField("X")
        .appendField(new FieldVariable(variables.defaultVarName, variables.validateVarName), "VAR1")
        .appendField("Y")
        .appendField(new FieldVariable(variables.defaultVarName2, variables.validateVarName), "VAR2")
        .appendField("S")
        .appendField(new FieldVariable(variables.defaultVarName3, variables.validateVarName), "VAR3")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('Nunchuck (103020009)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  getVars: function () {
    return variables.getAll(this, ["VAR1", "VAR2", "VAR3"]);
  },
  renameVar: function (oldName, newName) {
    return variables.rename(this, ["VAR1", "VAR2", "VAR3"], oldName, newName);
  },
};

blockDefs['grove_3axis_digital_acc'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.INPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C 3-Axis Dig. Acc.")
        .appendField(new BlocklyLib.FieldDropdown([
          ["1.5g", "15"],
          ["16g", "16"],
          ["400g", "400"]
        ]), "TYPE")
    this.appendDummyInput()
        .appendField("X")
        .appendField(new FieldVariable(variables.defaultVarName, variables.validateVarName), "VAR1")
        .appendField("Y")
        .appendField(new FieldVariable(variables.defaultVarName2, variables.validateVarName), "VAR2")
        .appendField("Z")
        .appendField(new FieldVariable(variables.defaultVarName3, variables.validateVarName), "VAR3")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('3-Axis Digital Acc +1.5g (101020039) 3-Axis Digital Acc +16g (101020054) 3-Axis Digital Acc +400g (101020071)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  getVars: function () {
    return variables.getAll(this, ["VAR1", "VAR2", "VAR3"]);
  },
  renameVar: function (oldName, newName) {
    return variables.rename(this, ["VAR1", "VAR2", "VAR3"], oldName, newName);
  },
};

blockDefs['grove_3axis_digital_acc_int'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.INPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C 3-Axis Dig. Acc.")
        .appendField(new BlocklyLib.FieldDropdown([
          ["1.5g", "15"],
          ["16g", "16"],
          ["400g", "400"]
        ]), "TYPE")
    this.appendDummyInput()
         .appendField(new BlocklyLib.FieldDropdown([
          ["Axis X", "X"],
          ["Axis Y", "Y"],
          ["Axis Z", "Z"]
         ]), "COMMAND")
        .appendField(msg.get("GROVE_SIGN"))
        .appendField(new FieldVariable(variables.defaultVarName, variables.validateVarName), "VARS")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.appendDummyInput()
        .appendField(msg.get("GROVE_INTEGER"))
        .appendField(new FieldVariable(variables.defaultVarName2, variables.validateVarName), "VARI")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('3-Axis Digital Acc +1.5g (101020039) 3-Axis Digital Acc +16g (101020054) 3-Axis Digital Acc +400g (101020071)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  getVars: function() {
    return variables.getAll(this, ["VARS", "VARI"]);
  },
  renameVar: function(oldName, newName) {
    return variables.rename(this, ["VARS", "VARI"], oldName, newName);
  },
};

blockDefs['grove_3axis_digital_compass'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.INPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C 3-Axis Dig. Compass")
        .appendField(new BlocklyLib.FieldDropdown([
          ["v1", "10"],
          ["v2", "20"]
        ]), "TYPE")
    this.appendDummyInput()
        .appendField("X")
        .appendField(new FieldVariable(variables.defaultVarName, variables.validateVarName), "VAR1")
        .appendField("Y")
        .appendField(new FieldVariable(variables.defaultVarName2, variables.validateVarName), "VAR2")
        .appendField("Z")
        .appendField(new FieldVariable(variables.defaultVarName3, variables.validateVarName), "VAR3")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('3-Axis Digital Compass v1 (101020034) 3-Axis Digital Compass v2 (101020492)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  getVars: function () {
    return variables.getAll(this, ["VAR1", "VAR2", "VAR3"]);
  },
  renameVar: function (oldName, newName) {
    return variables.rename(this, ["VAR1", "VAR2", "VAR3"], oldName, newName);
  },
};

blockDefs['grove_3axis_digital_compass_int'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.INPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C 3-Axis Dig. Compass")
        .appendField(new BlocklyLib.FieldDropdown([
          ["v1", "10"],
          ["v2", "20"]
        ]), "TYPE")
    this.appendDummyInput()
         .appendField(new BlocklyLib.FieldDropdown([
          ["Axis X", "X"],
          ["Axis Y", "Y"],
          ["Axis Z", "Z"]
         ]), "COMMAND")
        .appendField(msg.get("GROVE_SIGN"))
        .appendField(new FieldVariable(variables.defaultVarName, variables.validateVarName), "VARS")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.appendDummyInput()
        .appendField(msg.get("GROVE_INTEGER"))
        .appendField(new FieldVariable(variables.defaultVarName2, variables.validateVarName), "VARI")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('3-Axis Digital Compass v1 (101020034) 3-Axis Digital Compass v2 (101020492)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  getVars: function() {
    return variables.getAll(this, ["VARS", "VARI"]);
  },
  renameVar: function(oldName, newName) {
    return variables.rename(this, ["VARS", "VARI"], oldName, newName);
  },
};

blockDefs['grove_barometer_high_accuracy'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.INPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C Barometer High Accuracy");
    this.appendDummyInput()
         .appendField(new BlocklyLib.FieldDropdown([
          [msg.get("GROVE_TEMPERATURE"), "0x32"],
          [msg.get("GROVE_PRESSURE"), "0x30"],
          [msg.get("GROVE_ALTITUDE"), "0x31"]
         ]), "COMMAND")
        .appendField(msg.get("GROVE_SIGN"))
        .appendField(new FieldVariable(variables.defaultVarName, variables.validateVarName), "VARS")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.appendDummyInput()
        .appendField(msg.get("GROVE_INTEGER"))
        .appendField(new FieldVariable(variables.defaultVarName2, variables.validateVarName), "VARI")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.appendDummyInput()
        .appendField(msg.get("GROVE_REMAINDER"))
        .appendField(new FieldVariable(variables.defaultVarName3, variables.validateVarName), "VARR")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('Barometer High Accuracy (101020068)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  getVars: function() {
    return variables.getAll(this, ["VARS", "VARI", "VARR"]);
  },
  renameVar: function(oldName, newName) {
    return variables.rename(this, ["VARS", "VARI", "VARR"], oldName, newName);
  },
};

blockDefs['grove_temperature_humidity'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.INPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("Temperature Humidity")
        .appendField(new BlocklyLib.FieldDropdown(io.makeOutputPinList()), "PIN")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.appendDummyInput()
        .appendField(" T")
        .appendField(new FieldVariable(variables.defaultVarName, variables.validateVarName), "VART")
        .appendField("H")
        .appendField(new FieldVariable(variables.defaultVarName2, variables.validateVarName), "VARH")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip("Temperature & Humidity Sensor DHT11 (101020011)");  
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  getVars: function() {
    return variables.getAll(this, ["VART", "VARH"]);
  },
  renameVar: function(oldName, newName) {
    return variables.rename(this, ["VART", "VARH"], oldName, newName);
  },
  outputPins: "PIN"
};

blockDefs['grove_serial_lcd_print'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendValueInput("VALUE")
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("Serial LCD" + " " + msg.get("GROVE_PRINT"))
        .setCheck([constants.TYPES.VAL,constants.TYPES.STR]);
    this.appendDummyInput()
        .setAlign(BlocklyLib.ALIGN_RIGHT)
        .appendField(" ")
        .appendField(new BlocklyLib.FieldCheckbox("TRUE"), "ASCII")
        .appendField(msg.get("PRINT_NUMBER_AS_TEXT"));
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('Serial LCD (104020004)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  onchange: blockUtils.checkInputs
};

blockDefs['grove_serial_lcd_command'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("Serial LCD")
         .appendField(new BlocklyLib.FieldDropdown([
          [msg.get("SERIAL_LCD_CLEAR"),       "0x9F, 0x01"],
          [msg.get("SERIAL_LCD_MOVE_LINE_1"), "0x9F, 0x80"],
          [msg.get("SERIAL_LCD_MOVE_LINE_2"), "0x9F, 0xC0"],
          ["Backlight on", "0x9F, 0x81"],
          ["Backlight off", "0x9F, 0x80"]
         ]), "CODETAG")
   this.setInputsInline(true);
   this.setPreviousStatement(true, "null");
   this.setNextStatement(true, "null");
   this.setTooltip('Serial LCD (104020004)');
   this.customContextMenu = contextMenus.customContextMenu;
   blockUtils.newBlock(this);
  },
};

blockDefs['grove_rgb_lcd_print'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C LCD RGB")
    this.appendDummyInput()
        .setAlign(BlocklyLib.ALIGN_RIGHT)
        .appendField(" ")
        .appendField(new BlocklyLib.FieldCheckbox("TRUE"), "ASCII")
        .appendField(msg.get("PRINT_NUMBER_AS_TEXT"));
    this.appendValueInput("VALUE")
        .appendField(msg.get("GROVE_PRINT"))
        .setAlign(BlocklyLib.ALIGN_RIGHT)
        .setCheck([constants.TYPES.VAL,constants.TYPES.STR]);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('LCD RGB Backlight (104030001)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  onchange: blockUtils.checkInputs
};

blockDefs['grove_rgb_lcd_command'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C LCD RGB")
         .appendField(new BlocklyLib.FieldDropdown([
          [msg.get("SERIAL_LCD_CLEAR"),       "0x01"],
          [msg.get("SERIAL_LCD_MOVE_LINE_1"), "0x80"],
          [msg.get("SERIAL_LCD_MOVE_LINE_2"), "0xC0"],
          ["Blink on", "hi2cout 0x07, (0x17)\n\thi2cout 0x06, (0x7f)"],
          ["Blink off", "hi2cout 0x07, (0x00)\n\thi2cout 0x06, (0xff)"]
         ]), "CODETAG")
   this.setInputsInline(true);
   this.setPreviousStatement(true, "null");
   this.setNextStatement(true, "null");
   this.setTooltip('LCD RGB Backlight (104030001)');
   this.customContextMenu = contextMenus.customContextMenu;
   blockUtils.newBlock(this);
  },
};

blockDefs['grove_rgb_lcd_color'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C LCD RGB" + " " + msg.get("GROVE_COLOUR"))
        .appendField(new BlocklyLib.FieldColour('#ff0000'), "RGB");
   this.setInputsInline(true);
   this.setPreviousStatement(true, "null");
   this.setNextStatement(true, "null");
   this.setTooltip('LCD RGB Backlight (104030001)');
   this.customContextMenu = contextMenus.customContextMenu;
   blockUtils.newBlock(this);
  },
};

blockDefs['grove_oled_print'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C OLED")
        .appendField(new BlocklyLib.FieldDropdown([
          ["128x64", "0"],
          ["128x128", "1"],
          ["96x96", "2"]
        ]), "TYPE")
    this.appendDummyInput()
        .setAlign(BlocklyLib.ALIGN_RIGHT)
        .appendField(" ")
        .appendField(new BlocklyLib.FieldCheckbox("TRUE"), "ASCII")
        .appendField(msg.get("PRINT_NUMBER_AS_TEXT"));
    this.appendValueInput("VALUE")
        .appendField(msg.get("GROVE_PRINT"))
        .setCheck([constants.TYPES.VAL,constants.TYPES.STR])
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('OLED Display 1.12" V2 (101020452) OLED Display 0.96" (104030008)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  onchange: blockUtils.checkInputs
};

blockDefs['grove_oled_plot'] = {
  init: function () {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C OLED")
        .appendField(new BlocklyLib.FieldDropdown([
          ["128x64", "0"],
          ["128x128", "1"],
          ["96x96", "2"]
        ]), "TYPE")
    this.appendValueInput("ROW")
        .appendField(msg.get("GROVE_MOVE_TO") + " " + msg.get("GROVE_LINE"))
        .setCheck([constants.TYPES.VAL])
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.appendValueInput("COL")
        .appendField(msg.get("GROVE_COLUMN"))
        .setCheck([constants.TYPES.VAL])
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('OLED Display 1.12" V2 (101020452) OLED Display 0.96" (104030008)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  onchange: blockUtils.checkInputs
};

blockDefs['grove_oled_command'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C OLED")
        .appendField(new BlocklyLib.FieldDropdown([
          ["128x64", "0"],
          ["128x128", "1"],
          ["96x96", "2"]
        ]), "TYPE");
    this.appendDummyInput()
         .appendField(new BlocklyLib.FieldDropdown([
          [msg.get("SERIAL_LCD_CLEAR"),       "0xFF"],
          [msg.get("SERIAL_LCD_MOVE_LINE_1"), "0"],
          [msg.get("SERIAL_LCD_MOVE_LINE_2"), "1"],
          [msg.get("SERIAL_LCD_MOVE_LINE_3"), "2"],
          [msg.get("SERIAL_LCD_MOVE_LINE_4"), "3"],
          [msg.get("SERIAL_LCD_MOVE_LINE_5"), "4"],
          [msg.get("SERIAL_LCD_MOVE_LINE_6"), "5"],
          [msg.get("SERIAL_LCD_MOVE_LINE_7"), "6"],
          [msg.get("SERIAL_LCD_MOVE_LINE_8"), "7"]
         ]), "CODETAG")
         .setAlign(BlocklyLib.ALIGN_RIGHT);
   this.setInputsInline(false);
   this.setPreviousStatement(true, "null");
   this.setNextStatement(true, "null");
   this.setTooltip('OLED Display 1.12" V2 (101020452) OLED Display 0.96" (104030008)');
   this.customContextMenu = contextMenus.customContextMenu;
   blockUtils.newBlock(this);
  },
};

blockDefs['grove_4_digit_display'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("4 Digit Display")
        .appendField(new BlocklyLib.FieldDropdown(io.makeOutputPinList()), "PIN")
    this.appendValueInput("VALUE")
        .appendField(msg.get("GROVE_BRIGHTNESS"))
        .appendField(new BlocklyLib.FieldDropdown([
           ["8", "8"], ["7", "7"], ["6", "6"], ["5", "5"],
           ["4", "4"], ["3", "3"], ["2", "2"], ["1", "1"], [msg.get("OUTPUT_OFF_STATE"), "0"]
        ]), "BRIGHTNESS")
        .appendField("  " + "Value")
        .setCheck([constants.TYPES.VAL]);
   this.setInputsInline(false);
   this.setPreviousStatement(true, "null");
   this.setNextStatement(true, "null");
   this.setTooltip('4 Digit Display (104030003)');
   this.customContextMenu = contextMenus.customContextMenu;
   blockUtils.newBlock(this);
  },
  outputPins: "PIN",
  onchange: blockUtils.checkInputs
};

blockDefs['grove_led_bar'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("LED Bar")
        .appendField(new BlocklyLib.FieldDropdown(io.makeOutputPinList()), "PIN");
    this.appendDummyInput()
        .appendField(msg.get("GROVE_MODE"))
         .appendField(new BlocklyLib.FieldDropdown([
           [msg.get("GROVE_BAR_UP"), "0x00"], [msg.get("GROVE_BAR_DOWN"), "0x10"], [msg.get("GROVE_BAR_SINGLE"), "0x20"]
         ]), "MODE")
    this.appendValueInput("VALUE")
         .appendField(msg.get("GROVE_BRIGHTNESS"))
         .appendField(new BlocklyLib.FieldDropdown([
           ["8", "7"], ["7", "6"], ["6", "5"], ["5", "4"],
           ["4", "3"], ["3", "2"], ["2", "1"], ["1", "0"], [msg.get("OUTPUT_OFF_STATE"), "8"]
         ]), "BRIGHTNESS")
         .appendField("  " + "Value")
         .setCheck([constants.TYPES.VAL]);
   this.setInputsInline(false);
   this.setPreviousStatement(true, "null");
   this.setNextStatement(true, "null");
   this.setTooltip('LED Bar (104030002)');
   this.customContextMenu = contextMenus.customContextMenu;
   blockUtils.newBlock(this);
  },
  outputPins: "PIN",
  onchange: blockUtils.checkInputs
};

blockDefs['grove_circular_led'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("Circular LED")
        .appendField(new BlocklyLib.FieldDropdown(io.makeOutputPinList()), "PIN");
    this.appendDummyInput()
        .appendField(msg.get("GROVE_MODE"))
         .appendField(new BlocklyLib.FieldDropdown([
           [msg.get("GROVE_BAR_UP"), "0x00"], [msg.get("GROVE_BAR_DOWN"), "0x10"], [msg.get("GROVE_BAR_SINGLE"), "0x20"]
         ]), "MODE")
    this.appendValueInput("VALUE")
         .appendField(msg.get("GROVE_BRIGHTNESS"))
         .appendField(new BlocklyLib.FieldDropdown([
           ["8", "7"], ["7", "6"], ["6", "5"], ["5", "4"],
           ["4", "3"], ["3", "2"], ["2", "1"], ["1", "0"], [msg.get("OUTPUT_OFF_STATE"), "8"]
         ]), "BRIGHTNESS")
         .appendField("  " + msg.get("GROVE_VALUE"))
         .setCheck([constants.TYPES.VAL]);
   this.setInputsInline(false);
   this.setPreviousStatement(true, "null");
   this.setNextStatement(true, "null");
   this.setTooltip('Circular LED (104030013)');
   this.customContextMenu = contextMenus.customContextMenu;
   blockUtils.newBlock(this);
  },
  outputPins: "PIN",
  onchange: blockUtils.checkInputs
};

blockDefs['grove_bluetooth_send'] = {
  init: function () {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("Serial Bluetooth v3.0" + "-" + msg.get("GROVE_SEND"))
    this.appendValueInput("VALUE").setCheck([constants.TYPES.VAL, constants.TYPES.VAR]);
    this.setInputsInline(true);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('Serial Bluetooth Module v3.0 (113020008)');
    this.customContextMenu = contextMenus.customContextMenu;
    this.onchange = blockUtils.checkInputs;
    blockUtils.newBlock(this);
  },
  onchange: function() {
    io.checkHSerialType(this);
  }
};

blockDefs['grove_bluetooth_receive'] = {
  init: function () {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.SERIAL_INPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("Serial Bluetooth v3.0" + "-" + msg.get("GROVE_RECEIVE"))
        .appendField(new FieldVariable(variables.defaultVarName, variables.validateVarName), "NAME")
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('Serial Bluetooth Module v3.0 (113020008)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  getVars: function() {
    return variables.getAll(this, ["NAME"]);
  },
  renameVar: function(oldName, newName) {
    return variables.rename(this, ["NAME"], oldName, newName);
  },
  onchange: function() {
    io.checkHSerialType(this);
  }
};

blockDefs['grove_mp3_v2'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("Serial MP3 Player v2.0")
    this.appendValueInput("VALUE")
        .setCheck(constants.TYPES.VAL)
        .appendField(new BlocklyLib.FieldDropdown([
          [msg.get("OUTPUT_PLAY_PLAY_TUNE"),        "play"],
          [msg.get("OUTPUT_PLAY_STOP_TUNE"),        "stop"],
          [msg.get("OUTPUT_PLAY_PAUSE_TUNE"),       "pause"],
          [msg.get("OUTPUT_PLAY_RESUME_TUNE"),      "resume"],
          [msg.get("OUTPUT_PLAY_SET_VOLUME_TO"),    "volume"]
        ]), "COMMAND")
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true);
    this.setNextStatement(true);
    this.setTooltip('MP3 Player v2.0 (107020008)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  onchange: function() {
    if (!blockUtils.canSetWarning(this)) return;
    var warning;
    var cmd = this.getFieldValue('COMMAND');
    //value will be NULL if the no (optional) connected block is actually connected, only required for play/volume 
    var value = basicGenerator.valueToCode(this, 'VALUE', basicGenerator.ORDER_ATOMIC);
    if (cmd == "play") {
      if (!value) warning = msg.get("MISSING_BLOCKS");
      else if (value > 255) warning = msg.get("TUNE_VALUE");
    }
    else if (cmd == "volume") {
      if (!value) warning = msg.get("MISSING_BLOCKS");
      else if (value > 30) warning = msg.get("VOLUME_VALUE");
    }
    this.setWarningText(warning);
  }
};

blockDefs['grove_i2c_motor'] = {
  init: function() {
    this.setHelpUrl(constants.GROVE_URL);
    this.setColour(colors.OUTPUT);
    this.appendDummyInput()
        .appendField(new BlocklyLib.FieldImage("assets/seeed.ico", 16, 16, "*"))
        .appendField("I2C Motor Driver")   
        .appendField(new BlocklyLib.FieldDropdown([
          ["address 1111", "0x1E"],
          ["address 1110", "0x1C"],
          ["address 1010", "0x14"],
          ["mini", "mini"]
        ]), "TYPE");
    this.appendValueInput("SPEED1")
        .setCheck(constants.TYPES.VAL)
        .appendField(msg.get("MOTORS_MOTOR") + " 1")
        .appendField(new BlocklyLib.FieldDropdown([[msg.get("MOTORS_DIRECTION_FORWARD"), "F"], [msg.get("MOTORS_DIRECTION_BACKWARD"), "B"], [msg.get("MOTORS_DIRECTION_STOP"), "S"], [msg.get("MOTORS_DIRECTION_BRAKE"), "K"]]), "DIRECTION1")
        .appendField(msg.get("MOTORS_GENERIC_SPEED"))
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.appendValueInput("SPEED2")
        .setCheck(constants.TYPES.VAL)
        .appendField(msg.get("MOTORS_MOTOR") + " 2")
        .appendField(new BlocklyLib.FieldDropdown([[msg.get("MOTORS_DIRECTION_FORWARD"), "F"], [msg.get("MOTORS_DIRECTION_BACKWARD"), "B"], [msg.get("MOTORS_DIRECTION_STOP"), "S"], [msg.get("MOTORS_DIRECTION_BRAKE"), "K"]]), "DIRECTION2")
        .appendField(msg.get("MOTORS_GENERIC_SPEED"))
        .setAlign(BlocklyLib.ALIGN_RIGHT);
    this.setInputsInline(false);
    this.setPreviousStatement(true, "null");
    this.setNextStatement(true, "null");
    this.setTooltip('I2C Mini Motor Driver (105020010) I2C Motor Driver Dual (105020001)');
    this.customContextMenu = contextMenus.customContextMenu;
    blockUtils.newBlock(this);
  },
  onchange: function() {
    if (!blockUtils.canSetWarning(this)) return;
    var warning;
    var dir1 = this.getFieldValue('DIRECTION1');
    var dir2 = this.getFieldValue('DIRECTION2');
    //value will be NULL if the no (optional) connected block is actually connected, only required for moving 
    var speed1 = basicGenerator.valueToCode(this, 'SPEED1', basicGenerator.ORDER_ATOMIC);
    var speed2 = basicGenerator.valueToCode(this, 'SPEED2', basicGenerator.ORDER_ATOMIC);
    if (dir1 == "F" || dir1 == "B") {
      if (!speed1) warning = msg.get("MISSING_BLOCKS");
    }
    if (dir2 == "F" || dir2 == "B") {
      if (!speed2) warning = msg.get("MISSING_BLOCKS");
    }
    this.setWarningText(warning);
  }  
};



});
